@@ -50,7 +50,7 @@ class ImageNode(Node):
 
             2.path (str) : path to image file (Note that this node will also contain the data) \n
 
-            3.resource (ndarray, PIL Image) : OpenCV, PIL (Note that this node will also contain the data)\n
+            3.resource (ndarray, PIL Image,Open3D) : OpenCV, PIL (Note that this node will also contain the data)\n
 
             4.xmlPath (str) : Xml file path from Agisoft Metashape\n
 
@@ -132,9 +132,10 @@ class ImageNode(Node):
         Args:
             1. np.ndarray (OpenCV) \n
             2. PIL Image\n
+            3. Open3D Image
 
         Raises:
-            ValueError: Resource must be np.ndarray (OpenCV), PIL Image, XMP file(str) or XML file .
+            ValueError: Resource must be np.ndarray (OpenCV), PIL Image or Open3D Image.
         """
 
         if type(value) is np.ndarray : #OpenCV
@@ -307,40 +308,40 @@ class ImageNode(Node):
     #             return None
     #     return self.orientedBoundingBox
 
-    def get_virtual_image(self, geometries: o3d.geometry)-> o3d.cpu.pybind.geometry.Image:
-        pinholeCamera=self.get_pinhole_camera_parameters()
+    def get_virtual_image(self, geometries: o3d.geometry, downsampling:int=2)-> o3d.cpu.pybind.geometry.Image:
+        pinholeCamera=self.get_pinhole_camera_parameters(downsampling)
         if pinholeCamera is not None:
             return gt.generate_virtual_image(geometries,pinholeCamera)
         else:
             return None
 
-    def get_pinhole_camera_parameters(self) -> o3d.camera.PinholeCameraParameters():
+    def get_pinhole_camera_parameters(self, downsampling:int=2) -> o3d.camera.PinholeCameraParameters():
         param=o3d.camera.PinholeCameraParameters()
         if getattr(self,'cartesianTransform',None) is not None:
             param.extrinsic=np.linalg.inv(self.cartesianTransform)
-            param.intrinsic=self.get_intrinsic_camera_parameters()
+            param.intrinsic=self.get_intrinsic_camera_parameters(downsampling)
             self.pinholeCamera=param
             return self.pinholeCamera
         else:
             return None
 
-    def get_intrinsic_camera_parameters(self) -> o3d.camera.PinholeCameraIntrinsic():
+    def get_intrinsic_camera_parameters(self, downsampling:int=2) -> o3d.camera.PinholeCameraIntrinsic():
 
-        if (getattr(self,'imageWidth',None) is not None and
-            getattr(self,'imageHeight',None) is not None and
-            getattr(self,'focalLength35mm',None) is not None and
-            getattr(self,'resolutionUnit',None) is not None and
-            getattr(self,'xResolution',None) is not None and 
-            getattr(self,'yResolution',None) is not None):
-                width=math.floor(self.imageWidth/(2*self.resolutionUnit))
-                height=math.floor(self.imageHeight/(2*self.resolutionUnit)) 
-                fx=self.focalLength35mm*self.xResolution/self.resolutionUnit
-                fy=self.focalLength35mm*self.yResolution/self.resolutionUnit
-        else:
-            width=640
-            height=480
-            fx=400
-            fy=400
+        width=640
+        height=480
+        f=25
+
+        if getattr(self,'imageWidth',None) is not None:
+            width=int(self.imageWidth/downsampling)
+        if getattr(self,'imageHeight',None) is not None:
+            height=int(self.imageHeight/downsampling)
+        if getattr(self,'focalLength35mm',None) is not None:
+            f=self.focalLength35mm
+
+        pixX=width/36 #these are standard 35mm film properties
+        pixY=height/24 #these are standard 35mm film properties
+        fx=pixX*f
+        fy=pixY*f
 
         if (getattr(self,'principalPointU',None) is not None and
             getattr(self,'principalPointV',None) is not None ):
