@@ -95,7 +95,7 @@ class MeshNode (GeometryNode):
             pass
         elif self.get_path():
             self._resource   =  o3d.io.read_triangle_mesh(self.path)
-        return self.resource  
+        return self._resource  
 
     def save_resource(self, directory:str=None,extension :str = '.ply') ->bool:
         """Export the resource of the Node.\n
