@@ -23,7 +23,7 @@ from warnings import warn
 RDF_EXTENSIONS = [".ttl"]
 IMG_EXTENSION = [".jpg", ".png", ".JPG", ".PNG", ".JPEG"]
 MESH_EXTENSION = [".obj",".ply",".fbx" ]
-PCD_EXTENSION = [".pcd", ".e57",".pts", ".ply"]
+PCD_EXTENSION = [".pcd", ".e57",".pts", ".ply", '.xml']
 INT_ATTRIBUTES = ['pointCount','faceCount','e57Index'] #'label'
 FLOAT_ATTRIBUTES = ['xResolution','yResolution','imageWidth','imageHeight','focalLength35mm','principalPointU','principalPointV','accuracy']
 LIST_ATTRIBUTES =  ['distortionCoeficients']
@@ -813,6 +813,9 @@ def match_uri(attribute :str) -> URIRef:
     #XCR
     elif attribute in ['focalLength35mm','principalPointU','principalPointV','distortionCoeficients','gsd']:
         return  xcr[attribute]
+    #XCR
+    elif attribute in ['isDerivedFromGeometry']:
+        return  omg[attribute]
     #V4D
     else:
         return v4d[attribute]
