@@ -11,6 +11,7 @@ import open3d as o3d
 import copy
 import pye57 
 pye57.e57.SUPPORTED_POINT_FIELDS.update({'nor:normalX' : 'd','nor:normalY': 'd','nor:normalZ': 'd'})
+from PIL import Image
 
 import xml.etree.ElementTree as ET 
 import math
@@ -71,10 +72,11 @@ def cap_mesh(test):
     # append the cap to the sliced mesh
     capped = sliced + cap
 
-
 def mesh_to_trimesh(geometry: o3d.geometry) -> trimesh.Trimesh:
     """
     Convert open3D.geometry.TriangleMesh to trimesh.Trimesh
+
+    **NOTE**: Only vertex_colors are implemented instead of face_colors with textures.
     """
     if type(geometry) is o3d.geometry.OrientedBoundingBox or type(geometry) is o3d.geometry.AxisAlignedBoundingBox:
         geometry=box_to_mesh(geometry)
@@ -86,24 +88,37 @@ def mesh_to_trimesh(geometry: o3d.geometry) -> trimesh.Trimesh:
         vertex_normals=None
         face_colors=None
         vertex_colors=None
+        color_visuals=None
+        texture_visuals=None
+        uv=None
 
         if geometry.has_triangle_normals():
             face_normals=geometry.triangle_normals
         if geometry.has_vertex_normals():
             vertex_normals=geometry.vertex_normals
-        if geometry.has_triangle_uvs(): # this is probably not correct
-            face_colors=geometry.triangle_uvs
+        if geometry.has_triangle_uvs():
+            uv=np.asarray(geometry.triangle_uvs)
+        # if geometry.textures[1] is not None:
+        #     pass
+            # myarray=np.asarray(geometry.textures[1])
+            # PIL_image = Image.fromarray(np.uint8(myarray)).convert('RGB')
+            # material = trimesh.visual.texture.SimpleMaterial(image=PIL_image)
+            # texture_visuals=trimesh.visual.TextureVisuals(uv=uv, material=material, image=PIL_image)
+            # color_visuals=trimesh.visual.ColorVisuals(mesh=)
+            # color_visuals = trimesh.visual.TextureVisuals(uv=uv, image=PIL_image, material=material)            
         if geometry.has_vertex_colors():
-            vertex_colors=geometry.vertex_colors
-
+            colors=np.asarray(geometry.vertex_colors)*255
+            vertex_colors=vertex_colors=colors.astype(int)
+            
         try:
-            triMesh= trimesh.Trimesh(vertices, faces, face_normals=face_normals, vertex_normals=vertex_normals, face_colors=face_colors, vertex_colors=vertex_colors)
+            triMesh= trimesh.Trimesh(vertices, faces, face_normals=face_normals, vertex_normals=vertex_normals, vertex_colors=vertex_colors) #, visual=texture_visuals
+            
             return triMesh
         except:
             print('Open3D to Trimesh failed')
             return None
     else:
-        print('Incorrect geometry input. Only input o3d.geometry.TriangleMesh,o3d.geometry.AxisAlignedBoundingBox or o3d.geometry.OrientedBoundingBox or type(geometry)  ')
+        raise ValueError('Incorrect geometry input. Only input o3d.geometry.TriangleMesh,o3d.geometry.AxisAlignedBoundingBox or o3d.geometry.OrientedBoundingBox or type(geometry)  ')
         return None
 
 def crop_geometry_by_convex_hull(source:trimesh.Trimesh, cutters: List[trimesh.Trimesh], inside : Boolean = True ) -> trimesh.Trimesh:
@@ -175,6 +190,7 @@ def get_mesh_collisions_trimesh(sourceMesh: o3d.geometry.TriangleMesh, geometrie
     Returns:
         List[int]: indices
     """
+    # geometries=ut.item_to_list(geometries)
     if (type(sourceMesh) is o3d.cpu.pybind.geometry.TriangleMesh and len(sourceMesh.triangles) >0):
         myTrimesh=mesh_to_trimesh(sourceMesh)
         geometries=ut.item_to_list(geometries)
@@ -680,12 +696,14 @@ def expand_box (box: o3d.geometry.OrientedBoundingBox, u=5.0,v=5.0,w=5.0) -> o3d
     Returns:
         o3d.geometry.OrientedBoundingBox
     """    
+    if 'OrientedBoundingBox' not in str(type(box)):
+        raise ValueError('Only OrientedBoundingBox allowed.' )
     if (u<0 and abs(u)>= box.extent[0]) or (v<0 and abs(v)>=box.extent[1]) or (w<0 and abs(w)>=box.extent[2]):
         raise ValueError("can't schrink more than the extent of the box")
 
     center = box.get_center()
     orientation = box.R 
-    extent = box.extent + [u, w, v] 
+    extent = box.extent + [u,v,w] 
     return o3d.geometry.OrientedBoundingBox(center,orientation,extent) 
 
 def join_geometries(geometries) ->o3d.geometry:
@@ -714,6 +732,56 @@ def join_geometries(geometries) ->o3d.geometry:
         print('submit point clouds or meshes')
         return None
 
+def determine_percentage_of_coverage(sources: o3d.geometry, reference:o3d.geometry,threshold:float=0.1)-> np.array:
+
+    #if no list, list
+    sources=ut.item_to_list(sources)
+    sourcePCDs=[]
+    indentityArray=None
+    percentages=np.full((len(sources),1),0.0,dtype=float)
+
+    # sample o3d.geometry and create identitylist so to track the indices.
+    for i,source in enumerate(sources):        
+        if 'PointCloud' in str(type(source)) :
+            sourcePCD=source.voxel_down_sample(threshold)
+            indentityArray=np.vstack((indentityArray,np.full((len(sourcePCD.points), 1), i)))
+            sourcePCDs.append(sourcePCD)
+        elif 'TriangleMesh' in str(type(source)):
+            area=source.get_surface_area()
+            count=int(area/(threshold*threshold))
+            sourcePCD=source.sample_points_uniformly(number_of_points=count)
+            indentityArray=np.vstack((indentityArray,np.full((len(sourcePCD.points), 1), i)))
+            sourcePCDs.append(sourcePCD)
+        else:
+            raise ValueError('sources must be o3d.geometry (PointCloud or TriangleMesh)')
+
+    indentityArray=indentityArray.flatten()
+    indentityArray=np.delete(indentityArray,0)
+
+    # check whether source and reference are close together to minize calculations
+    # ind=gt.get_pcd_collisions(reference,sourcePCDs) # get_pcd collisions doesn't work well with complete inliers
+    ind=get_box_inliers(reference.get_oriented_bounding_box(), [geometry.get_oriented_bounding_box() for geometry in sourcePCDs])
+    if not ind:
+        return percentages
+    sourcePCDs=[sourcePCDs[i] for i in ind] 
+
+    #compute distances
+    joinedPCD=join_geometries(sourcePCDs)
+    distances=joinedPCD.compute_point_cloud_distance(reference)
+
+    #remove distances > threshold
+    ind=np.where(np.asarray(distances) <= threshold)[0]
+    if ind.size ==0:
+        return percentages
+    indexArray=[indentityArray[i] for i in ind.tolist()]
+
+    # count occurences
+    unique_elements, counts_elements = np.unique(indexArray, return_counts=True)
+    for i in unique_elements:
+        percentages[i]=counts_elements[i]/len(sourcePCDs[i].points)
+
+    return percentages
+
 def crop_geometry_by_distance(source: o3d.geometry, cutters, threshold : float =0.10) -> o3d.geometry.PointCloud:
     """"Select the portion of a pointcloud that lies within a range of another mesh/point cloud 
     
@@ -801,6 +869,50 @@ def get_box_inliers(sourceBox:o3d.geometry.OrientedBoundingBox, testBoxes: List[
     return list
 
 
+def get_mesh_inliers(sources:o3d.geometry, reference:o3d.geometry) -> List[int]:
+
+
+    sources=ut.item_to_list(sources)
+    inliers=[None]*len(sources)
+    #validate reference
+    if ('TriangleMesh' in str(type(reference)) and 
+        len(reference.triangles)>=2 and
+        reference.is_watertight()):    
+        pass
+    elif 'TriangleMesh' in str(type(reference)) and len(reference.triangles)>=2:
+        reference,_=reference.compute_convex_hull()
+    elif ('PointCloud' in str(type(reference)) and len(reference.points)>=3):
+        reference,_=reference.compute_convex_hull()
+    else: 
+        raise ValueError(' Only TriangleMesh and PointCloud allowed')
+    #add reference to scene
+    scene = o3d.t.geometry.RaycastingScene()
+    reference = o3d.t.geometry.TriangleMesh.from_legacy(reference)
+    _ = scene.add_triangles(reference)
+
+    for i,s in enumerate(sources):
+        #validate sources
+        if ('TriangleMesh' in str(type(s)) and 
+            len(s.triangles)>=2 and
+            s.is_watertight()): 
+            pass
+        elif 'TriangleMesh' in str(type(s)) and len(s.triangles)>=2:
+            s,_=sources[i].compute_convex_hull()
+        elif ('PointCloud' in str(type(reference)) and len(reference.points)>=3):
+            s,_=sources[i].compute_convex_hull()
+        else:
+            continue
+        
+        #sample geometries
+        pcd=s.sample_points_uniformly(10)
+        query_points = o3d.core.Tensor(np.asarray(pcd.points), dtype=o3d.core.Dtype.Float32)
+        occupancy = scene.compute_occupancy(query_points)
+        if np.any(occupancy.numpy()):
+            inliers[i]=True
+            continue
+        inliers[i]=False
+    return inliers
+
 def get_box_intersections(sourceBox:o3d.geometry.OrientedBoundingBox, testBoxes: List[o3d.geometry.OrientedBoundingBox])->List[int]:
     """" Return indices of testBoxes of which the geometry intersects with the sourceBox.
     2 oriented bounding boxes (A,B) overlap if the projection from B in the coordinate system of A on all the axes overlap. 
