@@ -391,7 +391,14 @@ class TestGeometryutils(unittest.TestCase):
         # pcd + mesh
         result4=gt.crop_geometry_by_distance(source=sourcepcd,cutters=sourceMesh)
         self.assertLess(len(result4.points),1200000 ) 
-        
+    
+    def test_determine_percentage_of_coverage(self):
+        reference=self.pcd
+        sources=[self.slabMesh,self.wallMesh,self.columnMesh,self.beamMesh]
+        percentages=gt.determine_percentage_of_coverage(sources=sources,reference=reference)
+        self.assertEqual(len(percentages), len(sources))
+        [self.assertGreater(value,0.0) for value in percentages]
+
     def test_show_geometries(self):
         # gt.show_geometries((self.bimMeshes + [self.pcd, self.mesh]), color = True)
         self.assertEqual(0,0)
